with CGI, Text_IO, Ada.Integer_Text_IO, Ada.Strings.Unbounded,
     Ada.Characters.Handling, Ada.Strings.Maps.Constants, Ustrings;
use  CGI, Text_IO, Ada.Integer_Text_IO, Ada.Strings.Unbounded,
     Ada.Characters.Handling, Ada.Strings.Maps.Constants, Ustrings;

procedure Search is
-- Search for a requested search string in a requested text file;
-- the request and reply use the Common Gateway Interface (CGI) to
-- an HTTP server, and then on to a user of a World Wide Web (WWW) browser.

-- If a search string _and_ file name is sent, a search result is returned.
-- Otherwise, the program will reply with a form to fill out.
-- If some information (such as the file to search) is provided,
-- the form is specialized returned to "remember" the previous values sent.

-- This program can search many different files, but each file _MUST_ be
-- listed in the file "srchlist". The format for the "srchlist" file
-- is a list of lines with the following format:
--   local_file_name,User_Name
-- Here's an example:
--   /public/addresses/phone,Phone List
-- Also, lines can begin with "#", in which case they are comment lines.

-- To run this program directly (without an HTTP server), set the
-- environment variable REQUEST_METHOD to "GET" and the variable
-- QUERY_STRING to either "" or "file=David&query=hello&casesensitive=no".

-- Returning specialized forms takes a little extra effort since
-- package CGI doesn't automatically do this; a higher-level interface
-- than package CGI could simplify handling partially completed forms.


  -- CHANGE THIS LINE TO WHEREVER YOUR SEARCH LIST FILE IS:
  Search_List_Filename : constant String := "/home/wheeler/cgi-bin/srchlist";


  Search_List : File_Type;

  function User_Name(S : Unbounded_String) return Unbounded_String is
    X : Natural;
  begin
    X := Index(S, ",");
    return To_Unbounded_String(Slice(S, X+1, Length(S)));
  end User_Name;

  function Real_File_Name(S : Unbounded_String) return String is
    Line : Unbounded_String;
  begin
    while not End_Of_File(Search_List) loop
      Ustrings.Get_Line(Search_List, Line);
      if Element(Line, 1) /= '#'  then
        if User_Name(Line) = S then
          return Slice(Line, 1, Index(Line, ",") - 1);
        end if;
      end if;
    end loop;
    return "";
  end Real_File_Name;

  procedure Put_Matches(Filename : String; Pattern : String;
                        Case_Sensitive : Boolean) is
    Found_Something : Boolean := False;
    Found_Here : Boolean := False;
    Line     : Unbounded_String;
    Match_To : Unbounded_String;
    Search_File : File_Type;
    Clean_Pattern : String(1 .. Pattern'Length);
    -- Given Filename, pattern, and case sensitivity, put matches.
    -- This is currently implemented in a slow, inefficient way, but it's
    -- sufficient for the purpose for non-monstrous files.
  begin
    if not Case_Sensitive then
      Clean_Pattern := To_Lower(Pattern);
    else
      Clean_Pattern := Pattern;
    end if;
    if Filename = "" then
      Put_Line("<P><I>Sorry, that's not a searchable file.</I><P>");
      return;
    end if;
    if Pattern = "" then
      Put_Line("<P><I>Sorry, empty patterns are not permitted.</I><P>");
      return;
    end if;
    Open(Search_File, In_File, Filename);

    while not End_Of_File(Search_File) loop
      Get_Line(Search_File, Line);
      if not Case_Sensitive then
        Match_To := Translate(Line, Lower_Case_Map);
        Found_Here := (Index(Match_To, Clean_Pattern) /= 0);
      else
        Found_Here := (Index(Line, Clean_Pattern) /= 0);
      end if;
      if Found_Here then
        Put_Line(Line);
        Found_Something := True;
      end if;
    end loop;

    if not Found_Something then
      Put_Line("<I>No matches found</I>");
    end if;
  exception
    when Name_Error =>
        Put_Line("<I>File to search is not available</I>");
  end Put_Matches;

  procedure Process_Query is
    User_File_To_Search : constant String := CGI.Value("file");
    File_To_Search : constant String := Real_File_Name(U(User_File_To_Search));
    Pattern    : constant String := Value("query");
    Case_Sensitive : Boolean := False;
    Case_Sensitivity : constant String := Value ("casesensitive");
  begin
    Put_HTML_Head("Query Result");
    Put_HTML_Heading("Query Result", 1);
    Put_Line(String'("<P>The search for <I>" & Value("query") & "</I>"));
    Put_Line(String'(" in file <I>" & Value("file") & "</I>"));

    if Case_Sensitivity = "yes" then
      Case_Sensitive := True;
      Put_Line(" in a case-sensitive manner");
    end if;

    Put_Line("produced the following result:<P>");
    Put_Line("<PRE>");
    Flush;
    Put_Matches(File_To_Search, Pattern, Case_Sensitive);
    Put_Line("</PRE>");
  end Process_Query;

  procedure Put_Select_List is
    Line : Unbounded_String;
    Number_Of_Options : Natural := 0;
    First_Option : Boolean := True;
  begin
    -- Put a Selection list of legal filenames out.

    -- Count the number of options (non-comment lines).
    while not End_Of_File(Search_List) loop
      Get_Line(Search_List, Line);
      if Element(Line, 1) /= '#'  then
        Number_Of_Options := Number_Of_Options + 1;
      end if;
    end loop;
    Reset(Search_List);

    Put("<SELECT NAME=""file"" SIZE=");
    Put(Number_Of_Options);
    Put_Line(">");

    while not End_Of_File(Search_List) loop
      Get_Line(Search_List, Line);
      if Element(Line, 1) /= '#'  then
        Put("<OPTION VALUE=""");
        Put(User_Name(Line));
        Put('"');
        if First_Option then
          Put(" SELECTED");
          First_Option := False;
        end if;
        Put('>');
        Put(User_Name(Line));
      end if;
    end loop;
    Put_Line("</SELECT> <P>");
  end Put_Select_List;

  function Open_Search_List return Boolean is
  begin
    Open(Search_List, In_File, Search_List_Filename);
    return True;
  exception
    when Name_Error =>
        Put_Error_Message("Search List File is not available");
        return False;
    when Others =>
        Put_Error_Message("Search List File could not be opened");
        return False;
  end Open_Search_List;

begin
  Put_CGI_Header;
  if not Open_Search_List then
    return; -- Can't open search list, don't go any further.
  end if;

  if Key_Exists("query") and Key_Exists("file") then
    Process_Query;
  else
    Put_HTML_Head("Text Search Form");
    Put_HTML_Heading("Text Search Form", 1);
    Put_Line("<P>You may search for a text phrase");
    Put_Line(" from any of the given files.<P>");


    Put_Line("<FORM METHOD=POST>");

    Put_Line("What do you want to search for:<P>");
    Put_Line("<INPUT NAME=""query"" SIZE=40");

    declare
       Query_String : constant String := CGI.Value ("query");
       File_Value   : constant String := CGI.Value ("file");
    begin
       if Query_String /= "" then
          Put(" VALUE=");
          Put(String'(Value("query")));
       end if;
       Put_Line("><P>");

       if Key_Exists("file") and File_Value /= "" then
          Put("<INPUT TYPE=""hidden"" NAME=""file"" VALUE=""");
          Put(String'(Value("file") & """>"));
          Put("<P>You will be searching file <I>");
          Put(String'(Value("file")));
          Put_Line("</I><P>");
       else
          Put_Line("Where do you want to search?<P>");
          Put_Select_List;
       end if;
    end;

    if Key_Exists("casesensitive") then
      Put_Line(String'("<INPUT TYPE=""hidden"" NAME=""casesensitive"" VALUE=""" &
                Value("casesensitive") & """>"));
    else
      Put_Line("Do you want this search to be case-sensitive?");
      Put_Line("<DL>");
      Put_Line("<DD><INPUT TYPE=""radio"" NAME=""casesensitive"" " &
               "VALUE=""yes""> <I>Yes.</I>");
      Put_Line("<DD><INPUT TYPE=""radio"" NAME=""casesensitive"" " &
               "VALUE=""no"" CHECKED> <I>No.</I>");
      Put_Line("</DL>");
    end if;


    Put_Line("<P> <INPUT TYPE=""submit"" VALUE=""Submit Query"">");
    Put_Line("<INPUT TYPE=""reset""> ");
    Put_Line("</FORM>");
  end if;

  Put_HTML_Tail;
  Close(Search_List);
end Search;

